Coverage for matplotlib\__init__.py: 0%

# Coverage for matplotlib\__init__.py : 0%

## 837 statements 0 run 837 missing 0 excluded

Hot-keys on this page

r m x p toggle line displays

j k next/prev highlighted chunk

0 (zero) top of page

1 (one) first highlighted chunk

1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 762 763 764 765 766 767 768 769 770 771 772 773 774 775 776 777 778 779 780 781 782 783 784 785 786 787 788 789 790 791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 806 807 808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 1056 1057 1058 1059 1060 1061 1062 1063 1064 1065 1066 1067 1068 1069 1070 1071 1072 1073 1074 1075 1076 1077 1078 1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 1095 1096 1097 1098 1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 1109 1110 1111 1112 1113 1114 1115 1116 1117 1118 1119 1120 1121 1122 1123 1124 1125 1126 1127 1128 1129 1130 1131 1132 1133 1134 1135 1136 1137 1138 1139 1140 1141 1142 1143 1144 1145 1146 1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 1158 1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 1172 1173 1174 1175 1176 1177 1178 1179 1180 1181 1182 1183 1184 1185 1186 1187 1188 1189 1190 1191 1192 1193 1194 1195 1196 1197 1198 1199 1200 1201 1202 1203 1204 1205 1206 1207 1208 1209 1210 1211 1212 1213 1214 1215 1216 1217 1218 1219 1220 1221 1222 1223 1224 1225 1226 1227 1228 1229 1230 1231 1232 1233 1234 1235 1236 1237 1238 1239 1240 1241 1242 1243 1244 1245 1246 1247 1248 1249 1250 1251 1252 1253 1254 1255 1256 1257 1258 1259 1260 1261 1262 1263 1264 1265 1266 1267 1268 1269 1270 1271 1272 1273 1274 1275 1276 1277 1278 1279 1280 1281 1282 1283 1284 1285 1286 1287 1288 1289 1290 1291 1292 1293 1294 1295 1296 1297 1298 1299 1300 1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 1314 1315 1316 1317 1318 1319 1320 1321 1322 1323 1324 1325 1326 1327 1328 1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 1346 1347 1348 1349 1350 1351 1352 1353 1354 1355 1356 1357 1358 1359 1360 1361 1362 1363 1364 1365 1366 1367 1368 1369 1370 1371 1372 1373 1374 1375 1376 1377 1378 1379 1380 1381 1382 1383 1384 1385 1386 1387 1388 1389 1390 1391 1392 1393 1394 1395 1396 1397 1398 1399 1400 1401 1402 1403 1404 1405 1406 1407 1408 1409 1410 1411 1412 1413 1414 1415 1416 1417 1418 1419 1420 1421 1422 1423 1424 1425 1426 1427 1428 1429 1430 1431 1432 1433 1434 1435 1436 1437 1438 1439 1440 1441 1442 1443 1444 1445 1446 1447 1448 1449 1450 1451 1452 1453 1454 1455 1456 1457 1458 1459 1460 1461 1462 1463 1464 1465 1466 1467 1468 1469 1470 1471 1472 1473 1474 1475 1476 1477 1478 1479 1480 1481 1482 1483 1484 1485 1486 1487 1488 1489 1490 1491 1492 1493 1494 1495 1496 1497 1498 1499 1500 1501 1502 1503 1504 1505 1506 1507 1508 1509 1510 1511 1512 1513 1514 1515 1516 1517 1518 1519 1520 1521 1522 1523 1524 1525 1526 1527 1528 1529 1530 1531 1532 1533 1534 1535 1536 1537 1538 1539 1540 1541 1542 1543 1544 1545 1546 1547 1548 1549 1550 1551 1552 1553 1554 1555 1556 1557 1558 1559 1560 1561 1562 1563 1564 1565 1566 1567 1568 1569 1570 1571 1572 1573 1574 1575 1576 1577 1578 1579 1580 1581 1582 1583 1584 1585 1586 1587 1588 1589 1590 1591 1592 1593 1594 1595 1596 1597 1598 1599 1600 1601 1602 1603 1604 1605 1606 1607 1608 1609 1610 1611 1612 1613 1614 1615 1616 1617 1618 1619 1620 1621 1622 1623 1624 1625 1626 1627 1628 1629 1630 1631 1632 1633 1634 1635 1636 1637 1638 1639 1640 1641 1642 1643 1644 1645 1646 1647 1648 1649 1650 1651 1652 1653 1654 1655 1656 1657 1658 1659 1660 1661 1662 1663 1664 1665 1666 1667 1668 1669 1670 1671 1672 1673 1674 1675 1676 1677 1678 1679 1680 1681 1682 1683 1684 1685 1686 1687 1688 1689 1690 1691 1692 1693 1694 1695 1696 1697 1698 1699 1700 1701 1702 1703 1704 1705 1706 1707 1708 1709 1710 1711 1712 1713 1714 1715 1716 1717 1718 1719 1720 1721 1722 1723 1724 1725 1726 1727 1728 1729 1730 1731 1732 1733 1734 1735 1736 1737 1738 1739 1740 1741 1742 1743 1744 1745 1746 1747 1748 1749 1750 1751 1752 1753 1754 1755 1756 1757 1758 1759 1760 1761 1762 1763 1764 1765 1766 1767 1768 1769 1770 1771 1772 1773 1774 1775 1776 1777 1778 1779 1780 1781 1782 1783 1784 1785 1786 1787 1788 1789 1790 1791 1792 1793 1794 1795 1796 1797 1798 1799 1800 1801 1802 1803 1804 1805 1806 1807 1808 1809 1810 1811 1812 1813 1814 1815 1816 1817 1818 1819 1820 1821 1822 1823 1824 1825 1826 1827 1828 1829 1830 1831 1832 1833 1834 1835 1836 1837 1838 1839 1840 1841 1842 1843 1844 1845 1846 1847 1848 1849 1850 1851 1852 1853 1854 1855 1856 1857 1858 1859 1860 1861 1862 1863 1864 1865 1866 1867 1868 1869 1870 1871 1872 1873 1874 1875 1876 1877 1878 1879 1880 1881 1882 | """ This is an object-oriented plotting library. A procedural interface is provided by the companion pyplot module, which may be imported directly, e.g.:: import matplotlib.pyplot as plt or using ipython:: ipython at your terminal, followed by:: In [1]: %matplotlib In [2]: import matplotlib.pyplot as plt at the ipython shell prompt. For the most part, direct use of the object-oriented library is encouraged when programming; pyplot is primarily for working interactively. The exceptions are the pyplot commands :func:`~matplotlib.pyplot.figure`, :func:`~matplotlib.pyplot.subplot`, :func:`~matplotlib.pyplot.subplots`, and :func:`~pyplot.savefig`, which can greatly simplify scripting. Modules include: :mod:`matplotlib.axes` defines the :class:`~matplotlib.axes.Axes` class. Most pylab commands are wrappers for :class:`~matplotlib.axes.Axes` methods. The axes module is the highest level of OO access to the library. :mod:`matplotlib.figure` defines the :class:`~matplotlib.figure.Figure` class. :mod:`matplotlib.artist` defines the :class:`~matplotlib.artist.Artist` base class for all classes that draw things. :mod:`matplotlib.lines` defines the :class:`~matplotlib.lines.Line2D` class for drawing lines and markers :mod:`matplotlib.patches` defines classes for drawing polygons :mod:`matplotlib.text` defines the :class:`~matplotlib.text.Text`, :class:`~matplotlib.text.TextWithDash`, and :class:`~matplotlib.text.Annotate` classes :mod:`matplotlib.image` defines the :class:`~matplotlib.image.AxesImage` and :class:`~matplotlib.image.FigureImage` classes :mod:`matplotlib.collections` classes for efficient drawing of groups of lines or polygons :mod:`matplotlib.colors` classes for interpreting color specifications and for making colormaps :mod:`matplotlib.cm` colormaps and the :class:`~matplotlib.image.ScalarMappable` mixin class for providing color mapping functionality to other classes :mod:`matplotlib.ticker` classes for calculating tick mark locations and for formatting tick labels :mod:`matplotlib.backends` a subpackage with modules for various gui libraries and output formats The base matplotlib namespace includes: :data:`~matplotlib.rcParams` a global dictionary of default configuration settings. It is initialized by code which may be overridden by a matplotlibrc file. :func:`~matplotlib.rc` a function for setting groups of rcParams values :func:`~matplotlib.use` a function for setting the matplotlib backend. If used, this function must be called immediately after importing matplotlib for the first time. In particular, it must be called **before** importing pylab (if pylab is imported). matplotlib was initially written by John D. Hunter (1968-2012) and is now developed and maintained by a host of others. Occasionally the internal documentation (python docstrings) will refer to MATLAB&reg;, a registered trademark of The MathWorks, Inc. """ from __future__ import absolute_import, division, print_function import six import atexit from collections import MutableMapping import contextlib import distutils.version import functools import io import inspect import itertools import locale import logging import os import re import shutil import stat import sys import tempfile import warnings # cbook must import matplotlib only within function # definitions, so it is safe to import from it here. from . import cbook from matplotlib.cbook import ( _backports, mplDeprecation, dedent, get_label, sanitize_sequence) from matplotlib.compat import subprocess from matplotlib.rcsetup import defaultParams, validate_backend, cycler import numpy from six.moves.urllib.request import urlopen from six.moves import reload_module as reload # Get the version from the _version.py versioneer file. For a git checkout, # this is computed based on the number of commits since the last tag. from ._version import get_versions __version__ = str(get_versions()['version']) del get_versions _log = logging.getLogger(__name__) __version__numpy__ = str('1.7.1') # minimum required numpy version __bibtex__ = r"""@Article{Hunter:2007, Author = {Hunter, J. D.}, Title = {Matplotlib: A 2D graphics environment}, Journal = {Computing In Science \& Engineering}, Volume = {9}, Number = {3}, Pages = {90--95}, abstract = {Matplotlib is a 2D graphics package used for Python for application development, interactive scripting, and publication-quality image generation across user interfaces and operating systems.}, publisher = {IEEE COMPUTER SOC}, year = 2007 }""" _python27 = (sys.version_info.major == 2 and sys.version_info.minor >= 7) _python34 = (sys.version_info.major == 3 and sys.version_info.minor >= 4) if not (_python27 or _python34): raise ImportError("Matplotlib requires Python 2.7 or 3.4 or later") if _python27: _log.addHandler(logging.NullHandler()) def compare_versions(a, b): "return True if a is greater than or equal to b" if a: if six.PY3: if isinstance(a, bytes): a = a.decode('ascii') if isinstance(b, bytes): b = b.decode('ascii') a = distutils.version.LooseVersion(a) b = distutils.version.LooseVersion(b) return a >= b else: return False try: import dateutil except ImportError: raise ImportError("Matplotlib requires dateutil") if not compare_versions(six.__version__, '1.10'): raise ImportError( "Matplotlib requires six>=1.10; you have %s" % six.__version__) try: import pyparsing except ImportError: raise ImportError("Matplotlib requires pyparsing") else: if not compare_versions(pyparsing.__version__, '2.0.1'): raise ImportError( "Matplotlib requires pyparsing>=2.0.1; you have %s" % pyparsing.__version__) if not compare_versions(numpy.__version__, __version__numpy__): raise ImportError( "Matplotlib requires numpy>=%s; you have %s" % ( __version__numpy__, numpy.__version__)) if not hasattr(sys, 'argv'): # for modpython sys.argv = [str('modpython')] def _is_writable_dir(p): """ p is a string pointing to a putative writable dir -- return True p is such a string, else False """ return os.access(p, os.W_OK) and os.path.isdir(p) _verbose_msg = """\ matplotlib.verbose is deprecated; Command line argument --verbose-LEVEL is deprecated. This functionality is now provided by the standard python logging library. To get more (or less) logging output: import logging logger = logging.getLogger('matplotlib') logger.set_level(logging.INFO)""" def _set_logger_verbose_level(level_str='silent', file_str='sys.stdout'): """ Use a --verbose-LEVEL level to set the logging level: """ levelmap = {'silent': logging.WARNING, 'helpful': logging.INFO, 'debug': logging.DEBUG, 'debug-annoying': logging.DEBUG, 'info': logging.INFO, 'warning': logging.WARNING} # Check that current state of logger isn't already more verbose # than the requested level. If it is more verbose, then leave more # verbose. newlev = levelmap[level_str] oldlev = _log.getEffectiveLevel() if newlev < oldlev: _log.setLevel(newlev) std = { 'sys.stdout': sys.stdout, 'sys.stderr': sys.stderr, } if file_str in std: fileo = std[file_str] else: fileo = sys.stdout try: fileo = open(file_str, 'w') # if this fails, we will just write to stdout except IOError: warnings.warn('could not open log file "{0}"' 'for writing. Check your ' 'matplotlibrc'.format(file_str)) console = logging.StreamHandler(fileo) console.setLevel(newlev) _log.addHandler(console) def _parse_commandline(): """ Check for --verbose-LEVEL type command line arguments and set logging level appropriately. """ levels = ('silent', 'helpful', 'debug', 'debug-annoying', 'info', 'warning') for arg in sys.argv[1:]: if arg.startswith('--verbose-'): level_str = arg[10:] # If it doesn't match one of ours, then don't even # bother noting it, we are just a 3rd-party library # to somebody else's script. if level_str in levels: _set_logger_verbose_level(level_str) _parse_commandline() class Verbose(object): """ A class to handle reporting. Set the fileo attribute to any file instance to handle the output. Default is sys.stdout """ levels = ('silent', 'helpful', 'debug', 'debug-annoying') vald = {level: i for i, level in enumerate(levels)} # parse the verbosity from the command line; flags look like # --verbose-silent or --verbose-helpful _commandLineVerbose = None for arg in sys.argv[1:]: if not arg.startswith('--verbose-'): continue level_str = arg[10:] # If it doesn't match one of ours, then don't even # bother noting it, we are just a 3rd-party library # to somebody else's script. if level_str in levels: _commandLineVerbose = level_str @cbook.deprecated("2.2", message=_verbose_msg) def __init__(self): self.set_level('silent') self.fileo = sys.stdout @cbook.deprecated("2.2", message=_verbose_msg) def set_level(self, level): 'set the verbosity to one of the Verbose.levels strings' if self._commandLineVerbose is not None: level = self._commandLineVerbose if level not in self.levels: warnings.warn('matplotlib: unrecognized --verbose-* string "%s".' ' Legal values are %s' % (level, self.levels)) else: self.level = level @cbook.deprecated("2.2", message=_verbose_msg) def set_fileo(self, fname): std = { 'sys.stdout': sys.stdout, 'sys.stderr': sys.stderr, } if fname in std: self.fileo = std[fname] else: try: fileo = open(fname, 'w') except IOError: raise ValueError('Verbose object could not open log file "{0}"' ' for writing.\nCheck your matplotlibrc ' 'verbose.fileo setting'.format(fname)) else: self.fileo = fileo @cbook.deprecated("2.2", message=_verbose_msg) def report(self, s, level='helpful'): """ print message s to self.fileo if self.level>=level. Return value indicates whether a message was issued """ if self.ge(level): print(s, file=self.fileo) return True return False @cbook.deprecated("2.2", message=_verbose_msg) def wrap(self, fmt, func, level='helpful', always=True): """ return a callable function that wraps func and reports it output through the verbose handler if current verbosity level is higher than level if always is True, the report will occur on every function call; otherwise only on the first time the function is called """ assert callable(func) def wrapper(*args, **kwargs): ret = func(*args, **kwargs) if (always or not wrapper._spoke): spoke = self.report(fmt % ret, level) if not wrapper._spoke: wrapper._spoke = spoke return ret wrapper._spoke = False wrapper.__doc__ = func.__doc__ return wrapper @cbook.deprecated("2.2", message=_verbose_msg) def ge(self, level): 'return true if self.level is >= level' return self.vald[self.level] >= self.vald[level] with warnings.catch_warnings(): warnings.simplefilter("ignore") verbose = Verbose() def _wrap(fmt, func, level=logging.DEBUG, always=True): """ return a callable function that wraps func and reports its output through logger if always is True, the report will occur on every function call; otherwise only on the first time the function is called """ assert callable(func) def wrapper(*args, **kwargs): ret = func(*args, **kwargs) if (always or not wrapper._spoke): _log.log(level, fmt % ret) spoke = True if not wrapper._spoke: wrapper._spoke = spoke return ret wrapper._spoke = False wrapper.__doc__ = func.__doc__ return wrapper def checkdep_dvipng(): try: s = subprocess.Popen([str('dvipng'), '-version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE) stdout, stderr = s.communicate() line = stdout.decode('ascii').split('\n')[1] v = line.split()[-1] return v except (IndexError, ValueError, OSError): return None def checkdep_ghostscript(): if checkdep_ghostscript.executable is None: if sys.platform == 'win32': # mgs is the name in miktex gs_execs = ['gswin32c', 'gswin64c', 'mgs', 'gs'] else: gs_execs = ['gs'] for gs_exec in gs_execs: try: s = subprocess.Popen( [str(gs_exec), '--version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE) stdout, stderr = s.communicate() if s.returncode == 0: v = stdout[:-1].decode('ascii') checkdep_ghostscript.executable = gs_exec checkdep_ghostscript.version = v except (IndexError, ValueError, OSError): pass return checkdep_ghostscript.executable, checkdep_ghostscript.version checkdep_ghostscript.executable = None checkdep_ghostscript.version = None # Deprecated, as it is unneeded and some distributions (e.g. MiKTeX 2.9.6350) # do not actually report the TeX version. @cbook.deprecated("2.1") def checkdep_tex(): try: s = subprocess.Popen([str('tex'), '-version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE) stdout, stderr = s.communicate() line = stdout.decode('ascii').split('\n')[0] pattern = r'3\.1\d+' match = re.search(pattern, line) v = match.group(0) return v except (IndexError, ValueError, AttributeError, OSError): return None def checkdep_pdftops(): try: s = subprocess.Popen([str('pdftops'), '-v'], stdout=subprocess.PIPE, stderr=subprocess.PIPE) stdout, stderr = s.communicate() lines = stderr.decode('ascii').split('\n') for line in lines: if 'version' in line: v = line.split()[-1] return v except (IndexError, ValueError, UnboundLocalError, OSError): return None def checkdep_inkscape(): if checkdep_inkscape.version is None: try: s = subprocess.Popen([str('inkscape'), '-V'], stdout=subprocess.PIPE, stderr=subprocess.PIPE) stdout, stderr = s.communicate() lines = stdout.decode('ascii').split('\n') for line in lines: if 'Inkscape' in line: v = line.split()[1] break checkdep_inkscape.version = v except (IndexError, ValueError, UnboundLocalError, OSError): pass return checkdep_inkscape.version checkdep_inkscape.version = None @cbook.deprecated("2.1") def checkdep_xmllint(): try: s = subprocess.Popen([str('xmllint'), '--version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE) stdout, stderr = s.communicate() lines = stderr.decode('ascii').split('\n') for line in lines: if 'version' in line: v = line.split()[-1] break return v except (IndexError, ValueError, UnboundLocalError, OSError): return None def checkdep_ps_distiller(s): if not s: return False flag = True gs_req = '8.60' gs_exec, gs_v = checkdep_ghostscript() if not compare_versions(gs_v, gs_req): flag = False warnings.warn(('matplotlibrc ps.usedistiller option can not be used ' 'unless ghostscript-%s or later is installed on your ' 'system') % gs_req) if s == 'xpdf': pdftops_req = '3.0' pdftops_req_alt = '0.9' # poppler version numbers, ugh pdftops_v = checkdep_pdftops() if compare_versions(pdftops_v, pdftops_req): pass elif (compare_versions(pdftops_v, pdftops_req_alt) and not compare_versions(pdftops_v, '1.0')): pass else: flag = False warnings.warn(('matplotlibrc ps.usedistiller can not be set to ' 'xpdf unless xpdf-%s or later is installed on ' 'your system') % pdftops_req) if flag: return s else: return False def checkdep_usetex(s): if not s: return False gs_req = '8.60' dvipng_req = '1.6' flag = True if _backports.which("tex") is None: flag = False warnings.warn('matplotlibrc text.usetex option can not be used unless ' 'TeX is installed on your system') dvipng_v = checkdep_dvipng() if not compare_versions(dvipng_v, dvipng_req): flag = False warnings.warn('matplotlibrc text.usetex can not be used with *Agg ' 'backend unless dvipng-%s or later is installed on ' 'your system' % dvipng_req) gs_exec, gs_v = checkdep_ghostscript() if not compare_versions(gs_v, gs_req): flag = False warnings.warn('matplotlibrc text.usetex can not be used unless ' 'ghostscript-%s or later is installed on your system' % gs_req) return flag def _get_home(): """Find user's home directory if possible. Otherwise, returns None. :see: http://mail.python.org/pipermail/python-list/2005-February/325395.html """ if six.PY2 and sys.platform == 'win32': path = os.path.expanduser(b"~").decode(sys.getfilesystemencoding()) else: path = os.path.expanduser("~") if os.path.isdir(path): return path for evar in ('HOME', 'USERPROFILE', 'TMP'): path = os.environ.get(evar) if path is not None and os.path.isdir(path): return path return None def _create_tmp_config_dir(): """ If the config directory can not be created, create a temporary directory. """ configdir = os.environ['MPLCONFIGDIR'] = ( tempfile.mkdtemp(prefix='matplotlib-')) atexit.register(shutil.rmtree, configdir) return configdir get_home = _wrap('$HOME=%s', _get_home, always=False) def _get_xdg_config_dir(): """ Returns the XDG configuration directory, according to the `XDG base directory spec <http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html>`_. """ path = os.environ.get('XDG_CONFIG_HOME') if path is None: path = get_home() if path is not None: path = os.path.join(path, '.config') return path def _get_xdg_cache_dir(): """ Returns the XDG cache directory, according to the `XDG base directory spec <http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html>`_. """ path = os.environ.get('XDG_CACHE_HOME') if path is None: path = get_home() if path is not None: path = os.path.join(path, '.cache') return path def _get_config_or_cache_dir(xdg_base): from matplotlib.cbook import mkdirs configdir = os.environ.get('MPLCONFIGDIR') if configdir is not None: configdir = os.path.abspath(configdir) if not os.path.exists(configdir): mkdirs(configdir) if not _is_writable_dir(configdir): return _create_tmp_config_dir() return configdir p = None h = get_home() if h is not None: p = os.path.join(h, '.matplotlib') if sys.platform.startswith(('linux', 'freebsd')): p = None if xdg_base is not None: p = os.path.join(xdg_base, 'matplotlib') if p is not None: if os.path.exists(p): if _is_writable_dir(p): return p else: try: mkdirs(p) except OSError: pass else: return p return _create_tmp_config_dir() def _get_configdir(): """ Return the string representing the configuration directory. The directory is chosen as follows: 1. If the MPLCONFIGDIR environment variable is supplied, choose that. 2a. On Linux, follow the XDG specification and look first in `$XDG_CONFIG_HOME`, if defined, or `$HOME/.config`. 2b. On other platforms, choose `$HOME/.matplotlib`. 3. If the chosen directory exists and is writable, use that as the configuration directory. 4. If possible, create a temporary directory, and use it as the configuration directory. 5. A writable directory could not be found or created; return None. """ return _get_config_or_cache_dir(_get_xdg_config_dir()) get_configdir = _wrap('CONFIGDIR=%s', _get_configdir, always=False) def _get_cachedir(): """ Return the location of the cache directory. The procedure used to find the directory is the same as for _get_config_dir, except using `$XDG_CACHE_HOME`/`~/.cache` instead. """ return _get_config_or_cache_dir(_get_xdg_cache_dir()) get_cachedir = _wrap('CACHEDIR=%s', _get_cachedir, always=False) def _decode_filesystem_path(path): if not isinstance(path, str): return path.decode(sys.getfilesystemencoding()) else: return path def _get_data_path(): 'get the path to matplotlib data' if 'MATPLOTLIBDATA' in os.environ: path = os.environ['MATPLOTLIBDATA'] if not os.path.isdir(path): raise RuntimeError('Path in environment MATPLOTLIBDATA not a ' 'directory') return path _file = _decode_filesystem_path(__file__) path = os.sep.join([os.path.dirname(_file), 'mpl-data']) if os.path.isdir(path): return path # setuptools' namespace_packages may highjack this init file # so need to try something known to be in matplotlib, not basemap import matplotlib.afm _file = _decode_filesystem_path(matplotlib.afm.__file__) path = os.sep.join([os.path.dirname(_file), 'mpl-data']) if os.path.isdir(path): return path # py2exe zips pure python, so still need special check if getattr(sys, 'frozen', None): exe_path = os.path.dirname(_decode_filesystem_path(sys.executable)) path = os.path.join(exe_path, 'mpl-data') if os.path.isdir(path): return path # Try again assuming we need to step up one more directory path = os.path.join(os.path.split(exe_path)[0], 'mpl-data') if os.path.isdir(path): return path # Try again assuming sys.path[0] is a dir not a exe path = os.path.join(sys.path[0], 'mpl-data') if os.path.isdir(path): return path raise RuntimeError('Could not find the matplotlib data files') def _get_data_path_cached(): if defaultParams['datapath'][0] is None: defaultParams['datapath'][0] = _get_data_path() return defaultParams['datapath'][0] get_data_path = _wrap('matplotlib data path %s', _get_data_path_cached, always=False) def get_py2exe_datafiles(): datapath = get_data_path() _, tail = os.path.split(datapath) d = {} for root, _, files in os.walk(datapath): # Need to explicitly remove cocoa_agg files or py2exe complains # NOTE I don't know why, but do as previous version if 'Matplotlib.nib' in files: files.remove('Matplotlib.nib') files = [os.path.join(root, filename) for filename in files] root = root.replace(tail, 'mpl-data') root = root[root.index('mpl-data'):] d[root] = files return list(d.items()) def matplotlib_fname(): """ Get the location of the config file. The file location is determined in the following order - `$PWD/matplotlibrc` - `$MATPLOTLIBRC` if it is a file (or a named pipe, which can be created e.g. by process substitution) - `$MATPLOTLIBRC/matplotlibrc` - `$MPLCONFIGDIR/matplotlibrc` - On Linux, - `$XDG_CONFIG_HOME/matplotlib/matplotlibrc` (if $XDG_CONFIG_HOME is defined) - or `$HOME/.config/matplotlib/matplotlibrc` (if $XDG_CONFIG_HOME is not defined) - On other platforms, - `$HOME/.matplotlib/matplotlibrc` if `$HOME` is defined. - Lastly, it looks in `$MATPLOTLIBDATA/matplotlibrc` for a system-defined copy. """ def gen_candidates(): yield os.path.join(six.moves.getcwd(), 'matplotlibrc') try: matplotlibrc = os.environ['MATPLOTLIBRC'] except KeyError: pass else: yield matplotlibrc yield os.path.join(matplotlibrc, 'matplotlibrc') yield os.path.join(_get_configdir(), 'matplotlibrc') yield os.path.join(get_data_path(), 'matplotlibrc') for fname in gen_candidates(): if os.path.exists(fname): st_mode = os.stat(fname).st_mode if stat.S_ISREG(st_mode) or stat.S_ISFIFO(st_mode): break # Return first candidate that is a file, or last candidate if none is # valid (in that case, a warning is raised at startup by `rc_params`). return fname # names of keys to deprecate # the values are a tuple of (new_name, f_old_2_new, f_new_2_old) # the inverse function may be `None` _deprecated_map = {} _deprecated_ignore_map = {'nbagg.transparent': 'figure.facecolor'} _obsolete_set = {'plugins.directory', 'text.dvipnghack'} # The following may use a value of None to suppress the warning. # do NOT include in _all_deprecated _deprecated_set = {'axes.hold', 'backend.qt4', 'backend.qt5'} _all_deprecated = set(itertools.chain( _deprecated_ignore_map, _deprecated_map, _obsolete_set)) class RcParams(MutableMapping, dict): """ A dictionary object including validation validating functions are defined and associated with rc parameters in :mod:`matplotlib.rcsetup` """ validate = dict((key, converter) for key, (default, converter) in six.iteritems(defaultParams) if key not in _all_deprecated) msg_depr = "%s is deprecated and replaced with %s; please use the latter." msg_depr_set = ("%s is deprecated. Please remove it from your " "matplotlibrc and/or style files.") msg_depr_ignore = "%s is deprecated and ignored. Use %s instead." msg_obsolete = ("%s is obsolete. Please remove it from your matplotlibrc " "and/or style files.") msg_backend_obsolete = ("The {} rcParam was deprecated in version 2.2. In" " order to force the use of a specific Qt binding," " either import that binding first, or set the " "QT_API environment variable.") # validate values on the way in def __init__(self, *args, **kwargs): self.update(*args, **kwargs) def __setitem__(self, key, val): try: if key in _deprecated_map: alt_key, alt_val, inverse_alt = _deprecated_map[key] warnings.warn(self.msg_depr % (key, alt_key), mplDeprecation) key = alt_key val = alt_val(val) elif key in _deprecated_set and val is not None: if key.startswith('backend'): warnings.warn(self.msg_backend_obsolete.format(key), mplDeprecation) else: warnings.warn(self.msg_depr_set % key, mplDeprecation) elif key in _deprecated_ignore_map: alt = _deprecated_ignore_map[key] warnings.warn(self.msg_depr_ignore % (key, alt), mplDeprecation) return elif key in _obsolete_set: warnings.warn(self.msg_obsolete % (key, ), mplDeprecation) return try: cval = self.validate[key](val) except ValueError as ve: raise ValueError("Key %s: %s" % (key, str(ve))) dict.__setitem__(self, key, cval) except KeyError: raise KeyError( '%s is not a valid rc parameter. See rcParams.keys() for a ' 'list of valid parameters.' % (key,)) def __getitem__(self, key): inverse_alt = None if key in _deprecated_map: alt_key, alt_val, inverse_alt = _deprecated_map[key] warnings.warn(self.msg_depr % (key, alt_key), mplDeprecation) key = alt_key elif key in _deprecated_ignore_map: alt = _deprecated_ignore_map[key] warnings.warn(self.msg_depr_ignore % (key, alt), mplDeprecation) key = alt elif key in _obsolete_set: warnings.warn(self.msg_obsolete % (key, ), mplDeprecation) return None val = dict.__getitem__(self, key) if inverse_alt is not None: return inverse_alt(val) else: return val def __repr__(self): import pprint class_name = self.__class__.__name__ indent = len(class_name) + 1 repr_split = pprint.pformat(dict(self), indent=1, width=80 - indent).split('\n') repr_indented = ('\n' + ' ' * indent).join(repr_split) return '{0}({1})'.format(class_name, repr_indented) def __str__(self): return '\n'.join('{0}: {1}'.format(k, v) for k, v in sorted(self.items())) def __iter__(self): """ Yield sorted list of keys. """ for k in sorted(dict.__iter__(self)): yield k def find_all(self, pattern): """ Return the subset of this RcParams dictionary whose keys match, using :func:`re.search`, the given ``pattern``. .. note:: Changes to the returned dictionary are *not* propagated to the parent RcParams dictionary. """ pattern_re = re.compile(pattern) return RcParams((key, value) for key, value in self.items() if pattern_re.search(key)) def rc_params(fail_on_error=False): """Return a :class:`matplotlib.RcParams` instance from the default matplotlib rc file. """ fname = matplotlib_fname() if not os.path.exists(fname): # this should never happen, default in mpl-data should always be found message = 'could not find rc file; returning defaults' ret = RcParams([(key, default) for key, (default, _) in six.iteritems(defaultParams) if key not in _all_deprecated]) warnings.warn(message) return ret return rc_params_from_file(fname, fail_on_error) URL_REGEX = re.compile(r'http://|https://|ftp://|file://|file:\\') def is_url(filename): """Return True if string is an http, ftp, or file URL path.""" return URL_REGEX.match(filename) is not None def _url_lines(f): # Compatibility for urlopen in python 3, which yields bytes. for line in f: yield line.decode('utf8') @contextlib.contextmanager def _open_file_or_url(fname): if is_url(fname): f = urlopen(fname) yield _url_lines(f) f.close() else: fname = os.path.expanduser(fname) encoding = locale.getpreferredencoding(do_setlocale=False) if encoding is None: encoding = "utf-8" with io.open(fname, encoding=encoding) as f: yield f _error_details_fmt = 'line #%d\n\t"%s"\n\tin file "%s"' def _rc_params_in_file(fname, fail_on_error=False): """Return :class:`matplotlib.RcParams` from the contents of the given file. Unlike `rc_params_from_file`, the configuration class only contains the parameters specified in the file (i.e. default values are not filled in). """ cnt = 0 rc_temp = {} with _open_file_or_url(fname) as fd: try: for line in fd: cnt += 1 strippedline = line.split('#', 1)[0].strip() if not strippedline: continue tup = strippedline.split(':', 1) if len(tup) != 2: error_details = _error_details_fmt % (cnt, line, fname) warnings.warn('Illegal %s' % error_details) continue key, val = tup key = key.strip() val = val.strip() if key in rc_temp: warnings.warn('Duplicate key in file "%s", line #%d' % (fname, cnt)) rc_temp[key] = (val, line, cnt) except UnicodeDecodeError: warnings.warn( ('Cannot decode configuration file %s with ' 'encoding %s, check LANG and LC_* variables') % (fname, locale.getpreferredencoding(do_setlocale=False) or 'utf-8 (default)')) raise config = RcParams() for key in ('verbose.level', 'verbose.fileo'): if key in rc_temp: val, line, cnt = rc_temp.pop(key) if fail_on_error: config[key] = val # try to convert to proper type or raise else: try: config[key] = val # try to convert to proper type or skip except Exception as msg: error_details = _error_details_fmt % (cnt, line, fname) warnings.warn('Bad val "%s" on %s\n\t%s' % (val, error_details, msg)) for key, (val, line, cnt) in six.iteritems(rc_temp): if key in defaultParams: if fail_on_error: config[key] = val # try to convert to proper type or raise else: try: config[key] = val # try to convert to proper type or skip except Exception as msg: error_details = _error_details_fmt % (cnt, line, fname) warnings.warn('Bad val "%s" on %s\n\t%s' % (val, error_details, msg)) elif key in _deprecated_ignore_map: warnings.warn('%s is deprecated. Update your matplotlibrc to use ' '%s instead.' % (key, _deprecated_ignore_map[key]), mplDeprecation) else: print(""" Bad key "%s" on line %d in %s. You probably need to get an updated matplotlibrc file from http://github.com/matplotlib/matplotlib/blob/master/matplotlibrc.template or from the matplotlib source distribution""" % (key, cnt, fname), file=sys.stderr) return config def rc_params_from_file(fname, fail_on_error=False, use_default_template=True): """Return :class:`matplotlib.RcParams` from the contents of the given file. Parameters ---------- fname : str Name of file parsed for matplotlib settings. fail_on_error : bool If True, raise an error when the parser fails to convert a parameter. use_default_template : bool If True, initialize with default parameters before updating with those in the given file. If False, the configuration class only contains the parameters specified in the file. (Useful for updating dicts.) """ config_from_file = _rc_params_in_file(fname, fail_on_error) if not use_default_template: return config_from_file iter_params = six.iteritems(defaultParams) with warnings.catch_warnings(): warnings.simplefilter("ignore", mplDeprecation) config = RcParams([(key, default) for key, (default, _) in iter_params if key not in _all_deprecated]) config.update(config_from_file) if config['datapath'] is None: config['datapath'] = get_data_path() if "".join(config['text.latex.preamble']): _log.info(""" ***************************************************************** You have the following UNSUPPORTED LaTeX preamble customizations: %s Please do not ask for support with these customizations active. ***************************************************************** """, '\n'.join(config['text.latex.preamble'])) _log.debug('loaded rc file %s', fname) return config # this is the instance used by the matplotlib classes rcParams = rc_params() if rcParams['examples.directory']: # paths that are intended to be relative to matplotlib_fname() # are allowed for the examples.directory parameter. # However, we will need to fully qualify the path because # Sphinx requires absolute paths. if not os.path.isabs(rcParams['examples.directory']): _basedir, _fname = os.path.split(matplotlib_fname()) # Sometimes matplotlib_fname() can return relative paths, # Also, using realpath() guarantees that Sphinx will use # the same path that matplotlib sees (in case of weird symlinks). _basedir = os.path.realpath(_basedir) _fullpath = os.path.join(_basedir, rcParams['examples.directory']) rcParams['examples.directory'] = _fullpath rcParamsOrig = rcParams.copy() with warnings.catch_warnings(): warnings.simplefilter("ignore", mplDeprecation) rcParamsDefault = RcParams([(key, default) for key, (default, converter) in six.iteritems(defaultParams) if key not in _all_deprecated]) rcParams['ps.usedistiller'] = checkdep_ps_distiller( rcParams['ps.usedistiller']) rcParams['text.usetex'] = checkdep_usetex(rcParams['text.usetex']) if rcParams['axes.formatter.use_locale']: locale.setlocale(locale.LC_ALL, '') def rc(group, **kwargs): """ Set the current rc params. Group is the grouping for the rc, e.g., for ``lines.linewidth`` the group is ``lines``, for ``axes.facecolor``, the group is ``axes``, and so on. Group may also be a list or tuple of group names, e.g., (*xtick*, *ytick*). *kwargs* is a dictionary attribute name/value pairs, e.g.,:: rc('lines', linewidth=2, color='r') sets the current rc params and is equivalent to:: rcParams['lines.linewidth'] = 2 rcParams['lines.color'] = 'r' The following aliases are available to save typing for interactive users: ===== ================= Alias Property ===== ================= 'lw' 'linewidth' 'ls' 'linestyle' 'c' 'color' 'fc' 'facecolor' 'ec' 'edgecolor' 'mew' 'markeredgewidth' 'aa' 'antialiased' ===== ================= Thus you could abbreviate the above rc command as:: rc('lines', lw=2, c='r') Note you can use python's kwargs dictionary facility to store dictionaries of default parameters. e.g., you can customize the font rc as follows:: font = {'family' : 'monospace', 'weight' : 'bold', 'size' : 'larger'} rc('font', **font) # pass in the font dict as kwargs This enables you to easily switch between several configurations. Use ``matplotlib.style.use('default')`` or :func:`~matplotlib.rcdefaults` to restore the default rc params after changes. """ aliases = { 'lw': 'linewidth', 'ls': 'linestyle', 'c': 'color', 'fc': 'facecolor', 'ec': 'edgecolor', 'mew': 'markeredgewidth', 'aa': 'antialiased', } if isinstance(group, six.string_types): group = (group,) for g in group: for k, v in six.iteritems(kwargs): name = aliases.get(k) or k key = '%s.%s' % (g, name) try: rcParams[key] = v except KeyError: raise KeyError(('Unrecognized key "%s" for group "%s" and ' 'name "%s"') % (key, g, name)) def rcdefaults(): """Restore the rc params from Matplotlib's internal defaults. See Also -------- rc_file_defaults : Restore the rc params from the rc file originally loaded by Matplotlib. matplotlib.style.use : Use a specific style file. Call ``style.use('default')`` to restore the default style. """ rcParams.clear() rcParams.update(rcParamsDefault) def rc_file_defaults(): """Restore the rc params from the original rc file loaded by Matplotlib. """ rcParams.update(rcParamsOrig) def rc_file(fname): """ Update rc params from file. """ rcParams.update(rc_params_from_file(fname)) class rc_context(object): """ Return a context manager for managing rc settings. This allows one to do:: with mpl.rc_context(fname='screen.rc'): plt.plot(x, a) with mpl.rc_context(fname='print.rc'): plt.plot(x, b) plt.plot(x, c) The 'a' vs 'x' and 'c' vs 'x' plots would have settings from 'screen.rc', while the 'b' vs 'x' plot would have settings from 'print.rc'. A dictionary can also be passed to the context manager:: with mpl.rc_context(rc={'text.usetex': True}, fname='screen.rc'): plt.plot(x, a) The 'rc' dictionary takes precedence over the settings loaded from 'fname'. Passing a dictionary only is also valid. For example a common usage is:: with mpl.rc_context(rc={'interactive': False}): fig, ax = plt.subplots() ax.plot(range(3), range(3)) fig.savefig('A.png', format='png') plt.close(fig) """ # While it may seem natural to implement rc_context using # contextlib.contextmanager, that would entail always calling the finally: # clause of the contextmanager (which restores the original rcs) including # during garbage collection; as a result, something like `plt.xkcd(); # gc.collect()` would result in the style being lost (as `xkcd()` is # implemented on top of rc_context, and nothing is holding onto context # manager except possibly circular references. def __init__(self, rc=None, fname=None): self._orig = rcParams.copy() try: if fname: rc_file(fname) if rc: rcParams.update(rc) except Exception: # If anything goes wrong, revert to the original rcs. dict.update(rcParams, self._orig) raise def __enter__(self): return self def __exit__(self, exc_type, exc_value, exc_tb): # No need to revalidate the original values. dict.update(rcParams, self._orig) _use_error_msg = """ This call to matplotlib.use() has no effect because the backend has already been chosen; matplotlib.use() must be called *before* pylab, matplotlib.pyplot, or matplotlib.backends is imported for the first time. The backend was *originally* set to {backend!r} by the following code: {tb} """ def use(arg, warn=True, force=False): """ Set the matplotlib backend to one of the known backends. The argument is case-insensitive. *warn* specifies whether a warning should be issued if a backend has already been set up. *force* is an **experimental** flag that tells matplotlib to attempt to initialize a new backend by reloading the backend module. .. note:: This function must be called *before* importing pyplot for the first time; or, if you are not using pyplot, it must be called before importing matplotlib.backends. If warn is True, a warning is issued if you try and call this after pylab or pyplot have been loaded. In certain black magic use cases, e.g. :func:`pyplot.switch_backend`, we are doing the reloading necessary to make the backend switch work (in some cases, e.g., pure image backends) so one can set warn=False to suppress the warnings. To find out which backend is currently set, see :func:`matplotlib.get_backend`. """ # Lets determine the proper backend name first if arg.startswith('module://'): name = arg else: # Lowercase only non-module backend names (modules are case-sensitive) arg = arg.lower() name = validate_backend(arg) # Check if we've already set up a backend if 'matplotlib.backends' in sys.modules: # Warn only if called with a different name if (rcParams['backend'] != name) and warn: import matplotlib.backends warnings.warn( _use_error_msg.format( backend=rcParams['backend'], tb=matplotlib.backends._backend_loading_tb), stacklevel=2) # Unless we've been told to force it, just return if not force: return need_reload = True else: need_reload = False # Store the backend name rcParams['backend'] = name # If needed we reload here because a lot of setup code is triggered on # module import. See backends/__init__.py for more detail. if need_reload: reload(sys.modules['matplotlib.backends']) try: use(os.environ['MPLBACKEND']) except KeyError: pass def get_backend(): """Return the name of the current backend.""" return rcParams['backend'] def interactive(b): """ Set interactive mode to boolean b. If b is True, then draw after every plotting command, e.g., after xlabel """ rcParams['interactive'] = b def is_interactive(): 'Return true if plot mode is interactive' return rcParams['interactive'] def tk_window_focus(): """Return true if focus maintenance under TkAgg on win32 is on. This currently works only for python.exe and IPython.exe. Both IDLE and Pythonwin.exe fail badly when tk_window_focus is on.""" if rcParams['backend'] != 'TkAgg': return False return rcParams['tk.window_focus'] default_test_modules = [ 'matplotlib.tests', 'matplotlib.sphinxext.tests', 'mpl_toolkits.tests', ] def _init_tests(): try: import faulthandler except ImportError: pass else: # CPython's faulthandler since v3.6 handles exceptions on Windows # https://bugs.python.org/issue23848 but until v3.6.4 it was # printing non-fatal exceptions https://bugs.python.org/issue30557 import platform if not (sys.platform == 'win32' and (3, 6) < sys.version_info < (3, 6, 4) and platform.python_implementation() == 'CPython'): faulthandler.enable() # The version of FreeType to install locally for running the # tests. This must match the value in `setupext.py` LOCAL_FREETYPE_VERSION = '2.6.1' from matplotlib import ft2font if (ft2font.__freetype_version__ != LOCAL_FREETYPE_VERSION or ft2font.__freetype_build_type__ != 'local'): warnings.warn( "Matplotlib is not built with the correct FreeType version to run " "tests. Set local_freetype=True in setup.cfg and rebuild. " "Expect many image comparison failures below. " "Expected freetype version {0}. " "Found freetype version {1}. " "Freetype build type is {2}local".format( LOCAL_FREETYPE_VERSION, ft2font.__freetype_version__, "" if ft2font.__freetype_build_type__ == 'local' else "not " ) ) try: import pytest try: from unittest import mock except ImportError: import mock except ImportError: print("matplotlib.test requires pytest and mock to run.") raise def test(verbosity=None, coverage=False, switch_backend_warn=True, recursionlimit=0, **kwargs): """run the matplotlib test suite""" _init_tests() if not os.path.isdir(os.path.join(os.path.dirname(__file__), 'tests')): raise ImportError("Matplotlib test data is not installed") old_backend = get_backend() old_recursionlimit = sys.getrecursionlimit() try: use('agg') if recursionlimit: sys.setrecursionlimit(recursionlimit) import pytest args = kwargs.pop('argv', []) provide_default_modules = True use_pyargs = True for arg in args: if any(arg.startswith(module_path) for module_path in default_test_modules): provide_default_modules = False break if os.path.exists(arg): provide_default_modules = False use_pyargs = False break if use_pyargs: args += ['--pyargs'] if provide_default_modules: args += default_test_modules if coverage: args += ['--cov'] if verbosity: args += ['-' + 'v' * verbosity] retcode = pytest.main(args, **kwargs) finally: if old_backend.lower() != 'agg': use(old_backend, warn=switch_backend_warn) if recursionlimit: sys.setrecursionlimit(old_recursionlimit) return retcode test.__test__ = False # pytest: this function is not a test def _replacer(data, key): """Either returns data[key] or passes data back. Also converts input data to a sequence as needed. """ # if key isn't a string don't bother if not isinstance(key, six.string_types): return (key) # try to use __getitem__ try: return sanitize_sequence(data[key]) # key does not exist, silently fall back to key except KeyError: return key _DATA_DOC_APPENDIX = """ .. note:: In addition to the above described arguments, this function can take a **data** keyword argument. If such a **data** argument is given, the following arguments are replaced by **data[<arg>]**: {replaced} """ def _add_data_doc(docstring, replace_names, replace_all_args): """Add documentation for a *data* field to the given docstring. Parameters ---------- docstring : str The input docstring. replace_names : list of strings or None The list of parameter names which arguments should be replaced by `data[name]`. If None, all arguments are replaced if they are included in `data`. replace_all_args : bool If True, all arguments in *args get replaced, even if they are not in replace_names. Returns ------- The augmented docstring. """ if docstring is None: docstring = '' else: docstring = dedent(docstring) _repl = "" if replace_names is None: _repl = "* All positional and all keyword arguments." else: if len(replace_names) != 0: _repl = "* All arguments with the following names: '{names}'." if replace_all_args: _repl += "\n * All positional arguments." _repl = _repl.format(names="', '".join(sorted(replace_names))) return docstring + _DATA_DOC_APPENDIX.format(replaced=_repl) def _preprocess_data(replace_names=None, replace_all_args=False, label_namer=None, positional_parameter_names=None): """ A decorator to add a 'data' kwarg to any a function. The signature of the input function must include the ax argument at the first position :: def foo(ax, *args, **kwargs) so this is suitable for use with Axes methods. Parameters ---------- replace_names : list of strings, optional, default: None The list of parameter names which arguments should be replaced by `data[name]`. If None, all arguments are replaced if they are included in `data`. replace_all_args : bool, default: False If True, all arguments in *args get replaced, even if they are not in replace_names. label_namer : string, optional, default: None The name of the parameter which argument should be used as label, if label is not set. If None, the label keyword argument is not set. positional_parameter_names : list of strings or callable, optional The full list of positional parameter names (excluding an explicit `ax`/'self' argument at the first place and including all possible positional parameter in `*args`), in the right order. Can also include all other keyword parameter. Only needed if the wrapped function does contain `*args` and (replace_names is not None or replace_all_args is False). If it is a callable, it will be called with the actual tuple of *args and the data and should return a list like above. NOTE: callables should only be used when the names and order of *args can only be determined at runtime. Please use list of names when the order and names of *args is clear before runtime! .. note:: decorator also converts MappingView input data to list. """ if replace_names is not None: replace_names = set(replace_names) def param(func): new_sig = None # signature is since 3.3 and wrapped since 3.2, but we support 3.4+. python_has_signature = python_has_wrapped = six.PY3 # if in a legacy version of python and IPython is already imported # try to use their back-ported signature if not python_has_signature and 'IPython' in sys.modules: try: import IPython.utils.signatures signature = IPython.utils.signatures.signature Parameter = IPython.utils.signatures.Parameter except ImportError: pass else: python_has_signature = True else: if python_has_signature: signature = inspect.signature Parameter = inspect.Parameter if not python_has_signature: arg_spec = inspect.getargspec(func) _arg_names = arg_spec.args _has_varargs = arg_spec.varargs is not None _has_varkwargs = arg_spec.keywords is not None else: sig = signature(func) _has_varargs = False _has_varkwargs = False _arg_names = [] params = list(sig.parameters.values()) for p in params: if p.kind is Parameter.VAR_POSITIONAL: _has_varargs = True elif p.kind is Parameter.VAR_KEYWORD: _has_varkwargs = True else: _arg_names.append(p.name) data_param = Parameter('data', Parameter.KEYWORD_ONLY, default=None) if _has_varkwargs: params.insert(-1, data_param) else: params.append(data_param) new_sig = sig.replace(parameters=params) # Import-time check: do we have enough information to replace *args? arg_names_at_runtime = False # there can't be any positional arguments behind *args and no # positional args can end up in **kwargs, so only *varargs make # problems. # http://stupidpythonideas.blogspot.de/2013/08/arguments-and-parameters.html if not _has_varargs: # all args are "named", so no problem # remove the first "ax" / self arg arg_names = _arg_names[1:] else: # Here we have "unnamed" variables and we need a way to determine # whether to replace a arg or not if replace_names is None: # all argnames should be replaced arg_names = None elif len(replace_names) == 0: # No argnames should be replaced arg_names = [] elif len(_arg_names) > 1 and (positional_parameter_names is None): # we got no manual parameter names but more than an 'ax' ... if len(replace_names - set(_arg_names[1:])) == 0: # all to be replaced arguments are in the list arg_names = _arg_names[1:] else: raise AssertionError( "Got unknown 'replace_names' and wrapped function " "{!r} uses '*args', need 'positional_parameter_names'" .format(func.__name__)) else: if positional_parameter_names is not None: if callable(positional_parameter_names): # determined by the function at runtime arg_names_at_runtime = True # so that we don't compute the label_pos at import time arg_names = [] else: arg_names = positional_parameter_names else: if replace_all_args: arg_names = [] else: raise AssertionError( "Got 'replace_names' and wrapped function {!r} " "uses *args, need 'positional_parameter_names' or " "'replace_all_args'".format(func.__name__)) # compute the possible label_namer and label position in positional # arguments label_pos = 9999 # bigger than all "possible" argument lists label_namer_pos = 9999 # bigger than all "possible" argument lists if (label_namer and # we actually want a label here ... arg_names and # and we can determine a label in *args ... (label_namer in arg_names)): # and it is in *args label_namer_pos = arg_names.index(label_namer) if "label" in arg_names: label_pos = arg_names.index("label") # Check the case we know a label_namer but we can't find it the # arg_names... Unfortunately the label_namer can be in **kwargs, # which we can't detect here and which results in a non-set label # which might surprise the user :-( if label_namer and not arg_names_at_runtime and not _has_varkwargs: if not arg_names: raise AssertionError( "label_namer {!r} can't be found as the parameter without " "'positional_parameter_names'".format(label_namer)) elif label_namer not in arg_names: raise AssertionError( "label_namer {!r} can't be found in the parameter names " "(known argnames: %s).".format(label_namer, arg_names)) else: # this is the case when the name is in arg_names pass @functools.wraps(func) def inner(ax, *args, **kwargs): # this is needed because we want to change these values if # arg_names_at_runtime==True, but python does not allow assigning # to a variable in a outer scope. So use some new local ones and # set them to the already computed values. _label_pos = label_pos _label_namer_pos = label_namer_pos _arg_names = arg_names label = None data = kwargs.pop('data', None) if data is None: # data validation args = tuple(sanitize_sequence(a) for a in args) else: if arg_names_at_runtime: # update the information about replace names and # label position _arg_names = positional_parameter_names(args, data) if (label_namer and # we actually want a label here ... _arg_names and # and we can find a label in *args (label_namer in _arg_names)): # and it is in *args _label_namer_pos = _arg_names.index(label_namer) if "label" in _arg_names: _label_pos = arg_names.index("label") # save the current label_namer value so that it can be used as # a label if _label_namer_pos < len(args): label = args[_label_namer_pos] else: label = kwargs.get(label_namer, None) # ensure a string, as label can't be anything else if not isinstance(label, six.string_types): label = None if (replace_names is None) or (replace_all_args is True): # all should be replaced args = tuple(_replacer(data, a) for j, a in enumerate(args)) else: # An arg is replaced if the arg_name of that position is # in replace_names ... if len(_arg_names) < len(args): raise RuntimeError( "Got more args than function expects") args = tuple(_replacer(data, a) if _arg_names[j] in replace_names else a for j, a in enumerate(args)) if replace_names is None: # replace all kwargs ... kwargs = dict((k, _replacer(data, v)) for k, v in six.iteritems(kwargs)) else: # ... or only if a kwarg of that name is in replace_names kwargs = dict((k, _replacer(data, v) if k in replace_names else v) for k, v in six.iteritems(kwargs)) # replace the label if this func "wants" a label arg and the user # didn't set one. Note: if the user puts in "label=None", it does # *NOT* get replaced! user_supplied_label = ( (len(args) >= _label_pos) or # label is included in args ('label' in kwargs) # ... or in kwargs ) if (label_namer and not user_supplied_label): if _label_namer_pos < len(args): kwargs['label'] = get_label(args[_label_namer_pos], label) elif label_namer in kwargs: kwargs['label'] = get_label(kwargs[label_namer], label) else: warnings.warn( "Tried to set a label via parameter %r in func %r but " "couldn't find such an argument.\n" "(This is a programming error, please report to " "the Matplotlib list!)" % (label_namer, func.__name__), RuntimeWarning, stacklevel=2) return func(ax, *args, **kwargs) inner.__doc__ = _add_data_doc(inner.__doc__, replace_names, replace_all_args) if not python_has_wrapped: inner.__wrapped__ = func if new_sig is not None: inner.__signature__ = new_sig return inner return param _log.debug('matplotlib version %s', __version__) _log.debug('interactive is %s', is_interactive()) _log.debug('platform is %s', sys.platform) _log.debug('loaded modules: %s', list(sys.modules))